
///////////////////////////////////////////////////////////
// Company         : Micro Digital                       //
// Address         : Office # C7 Raza Plaza              //
//                   DAV College Road                    //
//                   Rawalpindi,                         //
//                   Pakistan.                           //
// Programmed By   : Rashid Mehmood                      //
// Project Name    : 8051 Calculator using PC Serial     //
//                   Port                                //
// Crystal         : 24.000000 MHz                       //
// Microcontroller : AT89C51-C52-C55-S51-S52             //
///////////////////////////////////////////////////////////

// Header file for AT89x051 regiter definitions.
#include <AT89x51.H>
#include <stdio.h>
#include <stdlib.h>

// Header file for serial communication functions.
#include "Serial.h"

sbit ReadyLed = P2^0;
sbit OperatorLed = P2^1;
sbit Num1Led = P2^2;
sbit Num2Led = P2^3;

#define ADD 		'1'
#define SUBTRACT	'2'
#define MULTIPLY	'3'
#define DIVIDE		'4'

// This function waits until
// user enters a valid choice.
char GetOperator()
{
	char Operator = 0;
	uputs("Enter your choice\r\0");
	uputs("1) Add\r\0");
	uputs("2) Subtract\r\0");
	uputs("3) Multiply\r\0");
	uputs("4) Divide\r\0");

	while(Operator < '1' || Operator > '4')
		Operator = ugetc();
	return Operator;
}

// Main entry point.
void main()
{
	char sNum[15];		// Charactor string for multiple uses.
	long Num1;			// First number.
	long Num2;			// Second number.
	long Ans;			// Place to store result.
	char Operator = 0;	// Place to hold operation to be performed.
	char DivideByZero;
	InitSerial();		// Initialize serial channel of 8051.

	// Display title information.
	uputs("8051 Calculator using PC Serial Port\r\r");
	uputs("By    : Micro Digital\r\0");
	uputs("Web   : http://www.micro-digital.net\r\0");
	uputs("Email : udigital.solutions@gmail.com\r\0");
	uputs("Cell  : +92-332-555-8051\r\r\0");
	/////////////////////////////////////
	
	ReadyLed = 0;	// Switch on ready LED.

	while(1)
	{
		DivideByZero = 0;
		// Switch off all LED indicator except ready LED.
		OperatorLed = Num1Led = Num2Led = 1;
		Operator = GetOperator();
		OperatorLed = 0;	// Switch on operator received LED indicator.
		uputs("\rEnter 1st number (Max Length 4)\rNum1 : \0");
		ugets(sNum, 4);
		Num1 = atoi(sNum);
		Num1Led = 0;	// Switch on 1st number received LED indicator.
		uputs("Enter 2nd number (Max Length 4)\rNum2 : \0");
		ugets(sNum, 4);
		Num2 = atoi(sNum);
		Num2Led = 0;	// Switch on 1st number received LED indicator.
		switch(Operator)
		{
		case ADD:
			Ans = Num1 + Num2;
			break;

		case SUBTRACT:
			Ans = Num1 - Num2;
			break;

		case MULTIPLY:
			// Special case when result can
			// be greater so that it cannot
			// be stored without type casting.
			Ans = Num1 * Num2;
			break;

		case DIVIDE:
			if(Num2 == 0)
				DivideByZero = 1;
			else
				Ans = Num1 / Num2;
			break;
		}
		
		// If there is divide by zero error
		// then show error message on PC.
		if(DivideByZero)
		{
			uputs("Error: Divide By Zero\r\0");
		}
		else
		{
			// Prepare result to display
			// on PC.
			sprintf(sNum, "Ans  : %ld\r", Ans);
	
			// Display results on PC terminal software.
			uputs(sNum);
		}	
		// Hold microcontroller program and give 
		// user time to note results.
		uputs("Press any key to continue...\r\0");

		// Wait until user presses a key when ready
		// for next operation.
		ugetc();
	}
}

